#include <windows.h>
#include "lsapi.h"
#include "PyEmbed.h"
#include "PyScript.h"

vector<string> PyEmbed::scriptnames;

bool PyEmbed::addPythonPath (string newpath) {
	PyObject *pModuleName, *pModule, *pPythonPath, *pDict, *pNewPath;
   
	pModuleName = PyString_FromString ("sys");
    pModule = PyImport_Import(pModuleName);            //import that module into python
	Py_DECREF (pModuleName);                            //release the string (so the GC can deal with it)
    if (!pModule) {
        PyError::PyShowError ("PyScript Error", "Could not import \"sys\" for adding to PythonPath", "");     
        return false;
    }
	pDict = PyModule_GetDict(pModule);
	Py_DECREF (pModule);
	pPythonPath = PyDict_GetItemString(pDict, "path"); 
    if (!pPythonPath) {
            PyError::PyShowError ("PyScript Error", "Could not get sys.path", "");     
        return false;
    }
	if (PyList_Check(pPythonPath)) {
		pNewPath = PyString_FromString (newpath.c_str());   
		PyList_Append (pPythonPath, pNewPath);
        Py_DECREF(pNewPath);
		return true;
	}

    PyError::PyShowError ("PyScript Error", "sys.path is not a list", "");  
	return false;

}

bool PyEmbed::loadPythonScript (string scriptpath) {
    PyObject *pModuleName, *pModule, *pDict, *pFunc, *pArgs, *pResult;

    pModuleName = PyString_FromString(scriptpath.c_str());     //create a python string for a name of the module/file we want to run
    if (PyErr_Occurred()) {
        PyError::PyShowError ("PyScript Error", "Could not create a Python string from the scriptpath", "");        
        return false;
    }
    pModule = PyImport_Import(pModuleName);
    if (pModule == NULL) {
        string str = PyError::PyErrorDecode();
		MessageBox (NULL, str.c_str(), "pyscript", MB_OK);
 
        PyError::PyShowError (string ("PyScript Error"), string("Could not import the file ") + scriptpath, string(""));  
        return false;
    }

    pDict = PyModule_GetDict(pModule);           //get the dict of the module (listing of all symbols in there),
    Py_DECREF (pModule);
    pFunc = PyDict_GetItemString(pDict, "InitModuleEx");
    if (pFunc && PyCallable_Check(pFunc)) {      //test if that "pointer" points to a function and is callable
        pArgs = Py_BuildValue ("iis", PyScript::ParentWnd, PyScript::dllInst, scriptpath.c_str());
        pResult = PyInt_FromLong (0);
        pResult = PyObject_CallObject (pFunc, pArgs);
        Py_DECREF (pArgs);
        if (pResult == NULL) {
            PyError::PyShowError ("PyScript Error", "Could not call InitModuleEx", ""); 
            return false;
        }
        if (PyInt_AsLong(pResult) != 0) {
            PyError::PyShowError ("PyScript Error", "InitModule returned something else than 0", ""); 
            return false;
        }
        Py_DECREF (pResult);
        scriptnames.push_back (string (scriptpath.c_str()));
        return true;  
    } else {  
        PyError::PyShowError ("PyScript Error", "InitModule is not a callable function", ""); 
        return false;
    }
}

/* runs (optional) QuitModule for loaded scripts */
bool PyEmbed::unloadPythonScript (string scriptpath) {
    PyObject *pModuleName, *pModule, *pDict, *pFunc;

    pModuleName = PyString_FromString(scriptpath.c_str());     //create a python string for a name of the module/file we want to run
    if (PyErr_Occurred()) {
        PyError::PyShowError ("PyScript Error", "Could not create a Python string from the scriptpath for unloading", "");        
    }
    pModule = PyImport_Import(pModuleName);
    if (pModule == NULL) {
        PyError::PyShowError (string ("PyScript Error"), string("Could not import the file ") + scriptpath, string(""));  
    }
    pDict = PyModule_GetDict(pModule);           //get the dict of the module (listing of all symbols in there),
                                                 //pDict is a borrowed reference (no Py_DECREF)
    Py_DECREF (pModule);
    pFunc = PyDict_GetItemString(pDict, "QuitModule");
    if (pFunc && PyCallable_Check(pFunc)) {      //test if that "pointer" points to a function and is callable
        PyObject_CallObject (pFunc, NULL);
    }
    return true;
}
